@@ -84,16 +84,18 @@ typedef struct tds_compiletime_settings
 	const char *freetds_version;	/* release version of FreeTDS */
 	const char *sysconfdir;		/* location of freetds.conf */
 	const char *last_update;	/* latest software_version date among the modules */
-	int msdblib;		/* for MS style dblib */
-	int sybase_compat;	/* enable increased Open Client binary compatibility */
-	int threadsafe;		/* compile for thread safety default=no */
-	int libiconv;		/* search for libiconv in DIR/include and DIR/lib */
 	const char *tdsver;	/* TDS protocol version (4.2/4.6/5.0/7.0/7.1) 5.0 */
-	int iodbc;		/* build odbc driver against iODBC in DIR */
-	int unixodbc;		/* build odbc driver against unixODBC in DIR */
-	int openssl;		/* build against OpenSSL */
-	int gnutls;		/* build against GnuTLS */
-	int mars;		/* MARS enabled */
+	bool msdblib;		/* for MS style dblib */
+	bool sybase_compat;	/* enable increased Open Client binary compatibility */
+	bool threadsafe;		/* compile for thread safety default=no */
+	bool libiconv;		/* search for libiconv in DIR/include and DIR/lib */
+	bool iodbc;		/* build odbc driver against iODBC in DIR */
+	bool unixodbc;		/* build odbc driver against unixODBC in DIR */
+	bool openssl;		/* build against OpenSSL */
+	bool gnutls;		/* build against GnuTLS */
+	bool mars;		/* MARS enabled */
+	bool sspi;		/* SSPI enabled */
+	bool kerberos;		/* Kerberos enabled */
 } TDS_COMPILETIME_SETTINGS;
 
 /**
@@ -368,6 +370,7 @@ enum {
 	TDS_TYPEFLAG_BINARY   = 64,
 	TDS_TYPEFLAG_DATETIME = 128,
 	TDS_TYPEFLAG_NUMERIC  = 256,
+	TDS_TYPEFLAG_VARIANT  = 512,
 };
 
 extern const uint16_t tds_type_flags_ms[256];
@@ -379,6 +382,7 @@ extern const char *const tds_type_names[256];
 #define is_fixed_type(x)      ((tds_type_flags_ms[x] & TDS_TYPEFLAG_FIXED)    != 0)
 #define is_nullable_type(x)   ((tds_type_flags_ms[x] & TDS_TYPEFLAG_NULLABLE) != 0)
 #define is_variable_type(x)   ((tds_type_flags_ms[x] & TDS_TYPEFLAG_VARIABLE) != 0)
+#define is_variant_inner_type(x)   ((tds_type_flags_ms[x] & TDS_TYPEFLAG_VARIANT) != 0)
 
 
 #define is_blob_type(x)       ((x)==SYBTEXT || (x)==SYBIMAGE || (x)==SYBNTEXT)
@@ -445,7 +449,7 @@ bool is_tds_type_valid(int type)
 #define TDS_STR_HOST     "host"
 #define TDS_STR_PORT     "port"
 #define TDS_STR_TEXTSZ   "text size"
-/* for big endian hosts */
+/* for big endian hosts, obsolete, ignored */
 #define TDS_STR_EMUL_LE	"emulate little endian"
 #define TDS_STR_CHARSET	"charset"
 #define TDS_STR_CLCHARSET	"client charset"
@@ -465,6 +469,8 @@ bool is_tds_type_valid(int type)
 #define TDS_STR_ENCRYPTION_REQUIRE "require"
 /* Defines to enable optional GSSAPI delegation */
 #define TDS_GSSAPI_DELEGATION "enable gssapi delegation"
+/* Mutual authentication */
+#define TDS_STR_MUTUAL_AUTHENTICATION "mutual authentication"
 /* Kerberos realm name */
 #define TDS_STR_REALM	"realm"
 /* Kerberos SPN */
@@ -550,8 +556,8 @@ typedef struct tds_login
 
 	unsigned int bulk_copy:1;	/**< if bulk copy should be enabled */
 	unsigned int suppress_language:1;
-	unsigned int emul_little_endian:1;
 	unsigned int gssapi_use_delegation:1;
+	unsigned int mutual_authentication:1;
 	unsigned int use_ntlmv2:1;
 	unsigned int use_ntlmv2_specified:1;
 	unsigned int use_lanman:1;
@@ -630,7 +636,6 @@ typedef struct tds_bcpcoldata
 typedef TDSRET  tds_func_get_info(TDSSOCKET *tds, TDSCOLUMN *col);
 typedef TDSRET  tds_func_get_data(TDSSOCKET *tds, TDSCOLUMN *col);
 typedef TDS_INT tds_func_row_len(TDSCOLUMN *col);
-typedef unsigned tds_func_put_info_len(TDSSOCKET *tds, TDSCOLUMN *col);
 typedef TDSRET  tds_func_put_info(TDSSOCKET *tds, TDSCOLUMN *col);
 typedef TDSRET  tds_func_put_data(TDSSOCKET *tds, TDSCOLUMN *col, int bcp7);
 typedef int     tds_func_check(const TDSCOLUMN *col);
@@ -640,12 +645,6 @@ typedef struct tds_column_funcs
 	tds_func_get_info *get_info;
 	tds_func_get_data *get_data;
 	tds_func_row_len  *row_len;
-	/**
-	 * Returns metadata column information size.
-	 * \tds
-	 * \param col  column to send
-	 */
-	tds_func_put_info_len *put_info_len;
 	/**
 	 * Send metadata column information to server.
 	 * \tds
@@ -726,6 +725,7 @@ struct tds_column
 	unsigned char column_hidden:1;
 	unsigned char column_output:1;
 	unsigned char column_timestamp:1;
+	unsigned char column_computed:1;
 	TDS_UCHAR column_collation[5];
 
 	/* additional fields flags for compute results */
@@ -1048,6 +1048,8 @@ typedef struct tds_authentication
 {
 	uint8_t *packet;
 	int packet_len;
+	/* TDS_MSG_TOKEN type, for TDS5 */
+	uint16_t msg_type;
 	TDSRET (*free)(TDSCONNECTION* conn, struct tds_authentication * auth);
 	TDSRET (*handle_next)(TDSSOCKET * tds, struct tds_authentication * auth, size_t len);
 } TDSAUTHENTICATION;
@@ -1055,11 +1057,31 @@ typedef struct tds_authentication
 typedef struct tds_packet
 {
 	struct tds_packet *next;
-	short sid;
-	unsigned len, capacity;
+	uint16_t sid;
+
+#if ENABLE_ODBC_MARS
+	/**
+	 * Data before TDS data, currently can be 0 or sizeof(TDS72_SMP_HEADER)
+	 */
+	uint8_t data_start;
+#endif
+
+	/**
+	 * data length, this does not account SMP header, only TDS part
+	 */
+	unsigned data_len;
+	unsigned capacity;
 	unsigned char buf[1];
 } TDSPACKET;
 
+#if ENABLE_ODBC_MARS
+#define tds_packet_zero_data_start(pkt) do { (pkt)->data_start = 0; } while(0)
+#define tds_packet_get_data_start(pkt) ((pkt)->data_start)
+#else
+#define tds_packet_zero_data_start(pkt) do { } while(0)
+#define tds_packet_get_data_start(pkt) 0
+#endif
+
 typedef struct tds_poll_wakeup
 {
 	TDS_SYS_SOCKET s_signal, s_signaled;
@@ -1097,7 +1119,6 @@ struct tds_connection
 	TDS_UCHAR tds72_transaction[8];
 
 	TDS_CAPABILITIES capabilities;
-	unsigned int emul_little_endian:1;
 	unsigned int use_iconv:1;
 	unsigned int tds71rev1:1;
 	unsigned int pending_close:1;	/**< true is connection has pending closing (cursors or dynamic) */
@@ -1111,14 +1132,15 @@ struct tds_connection
 	TDSPACKET *send_packets;
 	unsigned send_pos, recv_pos;
 
-	tds_mutex list_mtx;
 #define BUSY_SOCKET ((TDSSOCKET*)(TDS_UINTPTR)1)
 #define TDSSOCKET_VALID(tds) (((TDS_UINTPTR)(tds)) > 1)
 	struct tds_socket **sessions;
 	unsigned num_sessions;
+#endif
+	tds_mutex list_mtx;
+
 	unsigned num_cached_packets;
 	TDSPACKET *packet_cache;
-#endif
 
 	int spid;
 	int client_spid;
@@ -1146,6 +1168,8 @@ struct tds_socket
 	TDSCONNECTION conn[1];
 #endif
 
+	void *parent;
+
 	/** Input buffer.
 	 * Points to receiving packet buffer.
 	 * As input buffer contains just the raw packet actually this pointer
@@ -1157,6 +1181,7 @@ struct tds_socket
 	 * Points to sending packet buffer.
 	 * Output buffer can contain additional data before the raw TDS packet
 	 * so this buffer can point some bytes after send_packet->buf.
+	 * Specifically this will point to send_packet->buf + send_packet->data_start.
 	 */
 	unsigned char *out_buf;
 
@@ -1171,11 +1196,31 @@ struct tds_socket
 	unsigned char in_flag;		/**< input buffer type */
 	unsigned char out_flag;		/**< output buffer type */
 
-	void *parent;
+	unsigned frozen;
+	/**
+	 * list of packets frozen, points to first one.
+	 * send_packet is the last packet in the list.
+	 */
+	TDSPACKET *frozen_packets;
 
 #if ENABLE_ODBC_MARS
-	short sid;
+	/** SID of MARS session.
+	 * ==0  Not in a MARS session or first session
+	 * >0   SID of MARS session valid.
+	 */
+	uint16_t sid;
+
+	/**
+	 * This condition will be signaled by the network thread on packet
+	 * received or sent for this session
+	 */
 	tds_condition packet_cond;
+
+	/**
+	 * Packet we are trying to send to network.
+	 * This field should be protected by conn->list_mtx
+	 */
+	TDSPACKET *sending_packet;
 	TDS_UINT recv_seq;
 	TDS_UINT send_seq;
 	TDS_UINT recv_wnd;
@@ -1200,14 +1245,14 @@ struct tds_socket
 	bool bulk_query;		/**< true is query sent was a bulk query so we need to switch state to QUERYING */
 	bool has_status; 		/**< true is ret_status is valid */
 	bool in_row;			/**< true if we are getting rows */
-	TDS_INT ret_status;     	/**< return status from store procedure */
-	TDS_STATE state;
 	volatile 
 	unsigned char in_cancel; 	/**< indicate we are waiting a cancel reply; discard tokens till acknowledge; 
 	1 mean we have to send cancel packet, 2 already sent. */
+	TDS_INT ret_status;     	/**< return status from store procedure */
+	TDS_STATE state;
 
-	TDS_INT8 rows_affected;		/**< rows updated/deleted/inserted/selected, TDS_NO_COUNT if not valid */
 	TDS_INT query_timeout;
+	TDS_INT8 rows_affected;		/**< rows updated/deleted/inserted/selected, TDS_NO_COUNT if not valid */
 
 	TDSDYNAMIC *cur_dyn;		/**< dynamic structure in use */
 
@@ -1261,7 +1306,7 @@ TDS_SERVER_TYPE tds_get_cardinal_type(TDS_SERVER_TYPE datatype, int usertype);
 TDSRET tds_iconv_open(TDSCONNECTION * conn, const char *charset, int use_utf16);
 void tds_iconv_close(TDSCONNECTION * conn);
 void tds_srv_charset_changed(TDSCONNECTION * conn, const char *charset);
-void tds7_srv_charset_changed(TDSCONNECTION * conn, int sql_collate, int lcid);
+void tds7_srv_charset_changed(TDSCONNECTION * conn, TDS_UCHAR collate[5]);
 int tds_iconv_alloc(TDSCONNECTION * conn);
 void tds_iconv_free(TDSCONNECTION * conn);
 TDSICONV *tds_iconv_from_collate(TDSCONNECTION * conn, TDS_UCHAR collate[5]);
@@ -1360,6 +1405,7 @@ TDSRET tds_submit_rollback(TDSSOCKET *tds, int cont);
 TDSRET tds_submit_commit(TDSSOCKET *tds, int cont);
 TDSRET tds_disconnect(TDSSOCKET * tds);
 size_t tds_quote_id(TDSSOCKET * tds, char *buffer, const char *id, int idlen);
+size_t tds_quote_id_rpc(TDSSOCKET * tds, char *buffer, const char *id, int idlen);
 size_t tds_quote_string(TDSSOCKET * tds, char *buffer, const char *str, int len);
 const char *tds_skip_comment(const char *s);
 const char *tds_skip_quoted(const char *s);
@@ -1511,11 +1557,51 @@ int tds_read_packet(TDSSOCKET * tds);
 TDSRET tds_write_packet(TDSSOCKET * tds, unsigned char final);
 #if ENABLE_ODBC_MARS
 int tds_append_cancel(TDSSOCKET *tds);
+TDSRET tds_append_syn(TDSSOCKET *tds);
 TDSRET tds_append_fin(TDSSOCKET *tds);
 #else
 int tds_put_cancel(TDSSOCKET * tds);
 #endif
 
+typedef struct tds_freeze {
+	/** which socket we refer to */
+	TDSSOCKET *tds;
+	/** first packet frozen */
+	TDSPACKET *pkt;
+	/** position in pkt */
+	unsigned pkt_pos;
+	/** length size (0, 1, 2 or 4) */
+	unsigned size_len;
+} TDSFREEZE;
+
+void tds_freeze(TDSSOCKET *tds, TDSFREEZE *freeze, unsigned size_len);
+size_t tds_freeze_written(TDSFREEZE *freeze);
+TDSRET tds_freeze_abort(TDSFREEZE *freeze);
+TDSRET tds_freeze_close(TDSFREEZE *freeze);
+TDSRET tds_freeze_close_len(TDSFREEZE *freeze, int32_t size);
+
+static void inline
+tds_set_current_send_packet(TDSSOCKET *tds, TDSPACKET *pkt)
+{
+	tds->send_packet = pkt;
+	tds->out_buf = pkt->buf + tds_packet_get_data_start(pkt);
+}
+
+/* Macros to allow some indentation of the packets.
+ *
+ * The 3 nested fake loops require some explanation:
+ * - first is to allows to declare variables;
+ * - second is to force using brackets;
+ * - third is to avoids that a break inside will skip the close.
+ */
+#define TDS_START_LEN_GENERIC(tds_socket, len) do { \
+	TDSFREEZE current_freeze[1]; \
+	tds_freeze((tds_socket), current_freeze, (len)); do { do
+#define TDS_END_LEN while(0); } while(tds_freeze_close(current_freeze), 0); } while(0);
+
+#define TDS_START_LEN_TINYINT(tds_socket) TDS_START_LEN_GENERIC(tds_socket, 1)
+#define TDS_START_LEN_USMALLINT(tds_socket) TDS_START_LEN_GENERIC(tds_socket, 2)
+#define TDS_START_LEN_UINT(tds_socket) TDS_START_LEN_GENERIC(tds_socket, 4)
 
 /* vstrbuild.c */
 TDSRET tds_vstrbuild(char *buffer, int buflen, int *resultlen, const char *text, int textlen, const char *formats, int formatlen,
@@ -1539,6 +1625,7 @@ TDSAUTHENTICATION * tds_gss_get_auth(TDSSOCKET * tds);
 #else
 TDSAUTHENTICATION * tds_sspi_get_auth(TDSSOCKET * tds);
 #endif
+TDSRET tds5_gss_send(TDSSOCKET *tds);
 
 
 /* random.c */
@@ -1547,7 +1634,12 @@ void tds_random_buffer(unsigned char *out, int len);
 
 /* sec_negotiate.c */
 TDSAUTHENTICATION * tds5_negotiate_get_auth(TDSSOCKET * tds);
-void tds5_negotiate_set_msg_type(TDSSOCKET * tds, TDSAUTHENTICATION * auth, unsigned msg_type);
+static inline
+void tds5_negotiate_set_msg_type(TDSAUTHENTICATION * tds_auth, unsigned msg_type)
+{
+	if (tds_auth)
+		tds_auth->msg_type = msg_type;
+}
 
 
 /* bulk.c */
