@@ -469,7 +469,7 @@ CS_RETCODE
 cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT * destfmt, CS_VOID * destdata, CS_INT * resultlen)
 {
 	TDS_SERVER_TYPE src_type, desttype;
-	int src_len, destlen, len, i = 0;
+	int src_len, destlen, len;
 	CONV_RESULT cres;
 	unsigned char *dest;
 	CS_RETCODE ret;
@@ -689,7 +689,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 	}
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_convert() calling tds_convert\n");
-	len = tds_convert(ctx->tds_ctx, src_type, (TDS_CHAR*) srcdata, src_len, desttype, &cres);
+	len = tds_convert(ctx->tds_ctx, src_type, srcdata, src_len, desttype, &cres);
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_convert() tds_convert returned %d\n", len);
 
@@ -737,8 +737,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 			destvc->len = len;
 			*resultlen = sizeof(*destvc);
 		}
-		for (i = len; i < destlen; i++)
-			dest[i] = '\0';
+		memset(dest + len, '\0', destlen - len);
 		break;
 	case SYBBIT:
 	case SYBBITN:
@@ -800,8 +799,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADBLANK\n");
 			/* strcpy here can lead to a small buffer overflow */
 			memcpy(dest, cres.c, len);
-			for (i = len; i < destlen; i++)
-				dest[i] = ' ';
+			memset(dest + len, ' ', destlen - len);
 			*resultlen = destlen;
 			break;
 
@@ -809,8 +807,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADNULL\n");
 			/* strcpy here can lead to a small buffer overflow */
 			memcpy(dest, cres.c, len);
-			for (i = len; i < destlen; i++)
-				dest[i] = '\0';
+			memset(dest + len, '\0', destlen - len);
 			*resultlen = destlen;
 			break;
 		case CS_FMT_UNUSED:
@@ -1411,12 +1408,15 @@ cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count)
  * Try to convert to a type we can handle
  */
 int
-_cs_convert_not_client(CS_CONTEXT *ctx, TDSCOLUMN *curcol, CONV_RESULT *convert_buffer, unsigned char **p_src)
+_cs_convert_not_client(CS_CONTEXT *ctx, const TDSCOLUMN *curcol, CONV_RESULT *convert_buffer, unsigned char **p_src)
 {
 	int ct_type;
-	TDS_SERVER_TYPE desttype;
+	TDS_SERVER_TYPE desttype, type = curcol->column_type;
 
-	switch (curcol->column_type) {
+	if (type == SYBVARIANT)
+		type = ((const TDSVARIANT *) curcol->column_data)->type;
+
+	switch (type) {
 	case SYBMSDATE:
 		desttype = SYBDATE;
 		ct_type = CS_DATE_TYPE;
@@ -1435,7 +1435,7 @@ _cs_convert_not_client(CS_CONTEXT *ctx, TDSCOLUMN *curcol, CONV_RESULT *convert_
 	}
 
 	if (convert_buffer) {
-		int len = tds_convert(ctx->tds_ctx, curcol->column_type, (TDS_CHAR *) *p_src,
+		int len = tds_convert(ctx->tds_ctx, type, *p_src,
 				      curcol->column_cur_size, desttype, convert_buffer);
 		if (len < 0)
 			return CS_ILLEGAL_TYPE; // TODO _csclient_msg ??
