@@ -44,7 +44,8 @@
 #include <freetds/iconv.h>
 #include <freetds/convert.h>
 #include <freetds/bytes.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
+#include <freetds/encodings.h>
 #include <replacements.h>
 #include <sybfront.h>
 #include <sybdb.h>
@@ -83,7 +84,8 @@ static TDSRET _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int off
 static TDSRET _bcp_no_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset);
 
 static int rtrim(char *, int);
-static STATUS _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error);
+static int rtrim_u16(uint16_t *str, int len, uint16_t space);
+static STATUS _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool skip);
 static int _bcp_readfmt_colinfo(DBPROCESS * dbproc, char *buf, BCP_HOSTCOLINFO * ci);
 static int _bcp_get_term_var(BYTE * pdata, BYTE * term, int term_len);
 
@@ -224,6 +226,8 @@ bcp_init(DBPROCESS * dbproc, const char *tblname, const char *hfile, const char
 
 	if (dbproc->hostfileinfo == NULL)
 		goto memory_error;
+	dbproc->hostfileinfo->maxerrs = 10;
+	dbproc->hostfileinfo->firstrow = 1;
 	if ((dbproc->hostfileinfo->hostfile = strdup(hfile)) == NULL)
 		goto memory_error;
 
@@ -552,9 +556,13 @@ bcp_control(DBPROCESS * dbproc, int field, DBINT value)
 	switch (field) {
 
 	case BCPMAXERRS:
+		if (value < 1)
+			value = 10;
 		dbproc->hostfileinfo->maxerrs = value;
 		break;
 	case BCPFIRST:
+		if (value < 1)
+			value = 1;
 		dbproc->hostfileinfo->firstrow = value;
 		break;
 	case BCPLAST:
@@ -755,11 +763,11 @@ _bcp_convert_out(DBPROCESS * dbproc, TDSCOLUMN *curcol, BCP_HOSTCOLINFO *hostcol
 			num->precision = 18;
 			num->scale = 0;
 		}
-		buflen = tds_convert(dbproc->tds_socket->conn->tds_ctx, srctype, (const TDS_CHAR *) src, srclen, hostcol->datatype, (CONV_RESULT *) num);
+		buflen = tds_convert(dbproc->tds_socket->conn->tds_ctx, srctype, src, srclen, hostcol->datatype, (CONV_RESULT *) num);
 		if (buflen > 0)
 			buflen = tds_numeric_bytes_per_prec[num->precision] + 2;
 	} else if (!is_variable_type(hostcol->datatype)) {
-		buflen = tds_convert(dbproc->tds_socket->conn->tds_ctx, srctype, (const TDS_CHAR *) src, srclen, hostcol->datatype, (CONV_RESULT *) (*p_data));
+		buflen = tds_convert(dbproc->tds_socket->conn->tds_ctx, srctype, src, srclen, hostcol->datatype, (CONV_RESULT *) (*p_data));
 	} else {
 		CONV_RESULT cr;
 
@@ -768,7 +776,7 @@ _bcp_convert_out(DBPROCESS * dbproc, TDSCOLUMN *curcol, BCP_HOSTCOLINFO *hostcol
 		 * because bcpcol->data_size is zero, so dbconvert() won't write anything,
 		 * and returns zero.
 		 */
-		buflen = tds_convert(dbproc->tds_socket->conn->tds_ctx, srctype, (const TDS_CHAR *) src, srclen, hostcol->datatype, (CONV_RESULT *) &cr);
+		buflen = tds_convert(dbproc->tds_socket->conn->tds_ctx, srctype, src, srclen, hostcol->datatype, (CONV_RESULT *) &cr);
 		if (buflen < 0)
 			return buflen;
 
@@ -991,7 +999,7 @@ _bcp_exec_out(DBPROCESS * dbproc, DBINT * rows_copied)
 	}
 	hostfile = NULL;
 
-	if (dbproc->hostfileinfo->firstrow > 0 && row_of_query < dbproc->hostfileinfo->firstrow) {
+	if (row_of_query + 1 < dbproc->hostfileinfo->firstrow) {
 		/*
 		 * The table which bulk-copy is attempting to
 		 * copy to a host-file is shorter than the
@@ -1071,6 +1079,51 @@ _bcp_convert_in(DBPROCESS *dbproc, TDS_SERVER_TYPE srctype, const TDS_CHAR *src,
 	return TDS_SUCCESS;
 }
 
+static void
+rtrim_bcpcol(TDSCOLUMN *bcpcol)
+{
+	/* trim trailing blanks from character data */
+	if (is_ascii_type(bcpcol->on_server.column_type)) {
+		/* A single NUL byte indicates an empty string. */
+		if (bcpcol->bcp_column_data->datalen == 1
+		    && bcpcol->bcp_column_data->data[0] == '\0') {
+			bcpcol->bcp_column_data->datalen = 0;
+			return;
+		}
+		bcpcol->bcp_column_data->datalen = rtrim((char *) bcpcol->bcp_column_data->data,
+								  bcpcol->bcp_column_data->datalen);
+		return;
+	}
+
+	/* unicode part */
+	if (is_unicode_type(bcpcol->on_server.column_type)) {
+		uint16_t *data, space;
+
+		if (!bcpcol->char_conv || bcpcol->char_conv->to.charset.min_bytes_per_char != 2)
+			return;
+
+		data = (uint16_t *) bcpcol->bcp_column_data->data;
+		/* A single NUL byte indicates an empty string. */
+		if (bcpcol->bcp_column_data->datalen == 2 && data[0] == 0) {
+			bcpcol->bcp_column_data->datalen = 0;
+			return;
+		}
+		switch (bcpcol->char_conv->to.charset.canonic) {
+		case TDS_CHARSET_UTF_16BE:
+		case TDS_CHARSET_UCS_2BE:
+			TDS_PUT_A2BE(&space, 0x20);
+			break;
+		case TDS_CHARSET_UTF_16LE:
+		case TDS_CHARSET_UCS_2LE:
+			TDS_PUT_A2LE(&space, 0x20);
+			break;
+		default:
+			return;
+		}
+		bcpcol->bcp_column_data->datalen = rtrim_u16(data, bcpcol->bcp_column_data->datalen, space);
+	}
+}
+
 /** 
  * \ingroup dblib_bcp_internal
  * \brief 
@@ -1083,7 +1136,7 @@ _bcp_convert_in(DBPROCESS *dbproc, TDS_SERVER_TYPE srctype, const TDS_CHAR *src,
  * \sa 	BCP_SETL(), bcp_batch(), bcp_bind(), bcp_colfmt(), bcp_colfmt_ps(), bcp_collen(), bcp_colptr(), bcp_columns(), bcp_control(), bcp_done(), bcp_exec(), bcp_getl(), bcp_init(), bcp_moretext(), bcp_options(), bcp_readfmt(), bcp_sendrow()
  */
 static STATUS
-_bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
+_bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool skip)
 {
 	TDSCOLUMN *bcpcol;
 	BCP_HOSTCOLINFO *hostcol;
@@ -1096,7 +1149,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
 
 	int i, collen, data_is_null;
 
-	tdsdump_log(TDS_DBG_FUNC, "_bcp_read_hostfile(%p, %p, %p)\n", dbproc, hostfile, row_error);
+	tdsdump_log(TDS_DBG_FUNC, "_bcp_read_hostfile(%p, %p, %p, %d)\n", dbproc, hostfile, row_error, skip);
 	assert(dbproc);
 	assert(hostfile);
 	assert(row_error);
@@ -1173,12 +1226,10 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
 		if (!data_is_null && hostcol->column_len >= 0) {
 			if (hostcol->column_len == 0)
 				data_is_null = 1;
-			else {
-				if (collen)
-					collen = (hostcol->column_len < collen) ? hostcol->column_len : collen;
-				else
-					collen = hostcol->column_len;
-			}
+			else if (collen)
+				collen = (hostcol->column_len < collen) ? hostcol->column_len : collen;
+			else
+				collen = hostcol->column_len;
 		}
 
 		tdsdump_log(TDS_DBG_FUNC, "prefix_len = %d collen = %d \n", hostcol->prefix_len, collen);
@@ -1243,7 +1294,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
 			 */
 		} else {	/* unterminated field */
 
-			coldata = tds_new(TDS_UCHAR, 1 + collen);
+			coldata = tds_new(TDS_CHAR, 1 + collen);
 			if (coldata == NULL) {
 				*row_error = TRUE;
 				dbperror(dbproc, SYBEMEM, errno);
@@ -1254,9 +1305,9 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
 			if (collen) {
 				/* 
 				 * Read and convert the data
-				 * TODO: Call tds_iconv_fread() instead of fread(3).  
+				 * TODO: Call tds_bcp_fread() instead of fread(3).
 				 *       The columns should each have their iconv cd set, and noncharacter data
-				 *       should have -1 as the iconv cd, causing tds_iconv_fread() to not attempt
+				 *       should have -1 as the iconv cd, causing tds_bcp_fread() to not attempt
 				 * 	 any conversion.  We do not need a datatype switch here to decide what to do.  
 				 *	 As of 0.62, this *should* actually work.  All that remains is to change the
 				 *	 call and test it. 
@@ -1273,7 +1324,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
 		 * At this point, however the field was read, however big it was, its address is coldata and its size is collen.
 		 */
 		tdsdump_log(TDS_DBG_FUNC, "Data read from hostfile: collen is now %d, data_is_null is %d\n", collen, data_is_null);
-		if (bcpcol) {
+		if (!skip && bcpcol) {
 			if (data_is_null) {
 				bcpcol->bcp_column_data->is_null = 1;
 				bcpcol->bcp_column_data->datalen = 0;
@@ -1292,17 +1343,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error)
 						    collen, (TDS_INT8) col_start);
 				}
 
-				/* trim trailing blanks from character data */
-				if (is_ascii_type(bcpcol->on_server.column_type)) {
-					/* A single NUL byte indicates an empty string. */
-					if (bcpcol->bcp_column_data->datalen == 1
-					    && bcpcol->bcp_column_data->data[0] == '\0') {
-						bcpcol->bcp_column_data->datalen = 0;
-					} else {
-						bcpcol->bcp_column_data->datalen = rtrim((char *) bcpcol->bcp_column_data->data,
-												  bcpcol->bcp_column_data->datalen);
-					}
-				}
+				rtrim_bcpcol(bcpcol);
 			}
 #if USING_SYBEBCNN
 			if (!hostcol->column_error) {
@@ -1420,15 +1461,25 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 	row_of_hostfile = 0;
 	rows_written_so_far = 0;
 
-	row_start = ftello(hostfile);
 	row_error_count = 0;
-	row_error = 0;
 	dbproc->bcpinfo->parent = dbproc;
 
-	while ((ret=_bcp_read_hostfile(dbproc, hostfile, &row_error)) == MORE_ROWS) {
+	for (;;) {
+		bool skip;
+
+		row_start = ftello(hostfile);
+		row_error = 0;
 
 		row_of_hostfile++;
 
+		if (row_of_hostfile > MAX(dbproc->hostfileinfo->lastrow, 0x7FFFFFFF))
+			break;
+
+		skip = dbproc->hostfileinfo->firstrow > row_of_hostfile;
+		ret = _bcp_read_hostfile(dbproc, hostfile, &row_error, skip);
+		if (ret != MORE_ROWS)
+			break;
+
 		if (row_error) {
 			int count;
 
@@ -1496,37 +1547,34 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 				}
 			}
 			row_error_count++;
-			if (row_error_count > dbproc->hostfileinfo->maxerrs)
+			if (row_error_count >= dbproc->hostfileinfo->maxerrs)
 				break;
-		} else {
-			if (dbproc->hostfileinfo->firstrow <= row_of_hostfile && 
-							      row_of_hostfile <= MAX(dbproc->hostfileinfo->lastrow, 0x7FFFFFFF)) {
-
-				if (TDS_SUCCEED(tds_bcp_send_record(dbproc->tds_socket, dbproc->bcpinfo, _bcp_no_get_col_data, NULL, 0))) {
-			
-					rows_written_so_far++;
-	
-					if (dbproc->hostfileinfo->batch > 0 && rows_written_so_far == dbproc->hostfileinfo->batch) {
-						if (TDS_FAILED(tds_bcp_done(tds, &rows_written_so_far))) {
-							if (errfile)
-								fclose(errfile);
-							fclose(hostfile);
-							return FAIL;
-						}
-							
-						*rows_copied += rows_written_so_far;
-						rows_written_so_far = 0;
-	
-						dbperror(dbproc, SYBEBBCI, 0); /* batch copied to server */
-	
-						tds_bcp_start(tds, dbproc->bcpinfo);
-					}
-				}
-			} 
+			continue;
 		}
 
-		row_start = ftello(hostfile);
-		row_error = 0;
+		if (skip)
+			continue;
+
+		if (TDS_SUCCEED(tds_bcp_send_record(dbproc->tds_socket, dbproc->bcpinfo, _bcp_no_get_col_data, NULL, 0))) {
+
+			rows_written_so_far++;
+
+			if (dbproc->hostfileinfo->batch > 0 && rows_written_so_far == dbproc->hostfileinfo->batch) {
+				if (TDS_FAILED(tds_bcp_done(tds, &rows_written_so_far))) {
+					if (errfile)
+						fclose(errfile);
+					fclose(hostfile);
+					return FAIL;
+				}
+
+				*rows_copied += rows_written_so_far;
+				rows_written_so_far = 0;
+
+				dbperror(dbproc, SYBEBBCI, 0); /* batch copied to server */
+
+				tds_bcp_start(tds, dbproc->bcpinfo);
+			}
+		}
 	}
 	
 	if( row_error_count == 0 && row_of_hostfile < dbproc->hostfileinfo->firstrow ) {
@@ -2036,7 +2084,7 @@ bcp_bind(DBPROCESS * dbproc, BYTE * varaddr, int prefixlen, DBINT varlen,
 						terminator, termlen, dbprtype(db_vartype), table_column);
 	CHECK_CONN(FAIL);
 	CHECK_PARAMETER(dbproc->bcpinfo, SYBEBCPI, FAIL);
-	DBPERROR_RETURN(!is_tds_type_valid(db_vartype), SYBEUDTY);
+	DBPERROR_RETURN(db_vartype != 0 && !is_tds_type_valid(db_vartype), SYBEUDTY);
 	vartype = (TDS_SERVER_TYPE) db_vartype;
 
 	if (dbproc->hostfileinfo != NULL) {
@@ -2274,6 +2322,17 @@ rtrim(char *str, int len)
 	return (int)(1 + p - str);
 }
 
+static int
+rtrim_u16(uint16_t *str, int len, uint16_t space)
+{
+	uint16_t *p = str + len / 2 - 1;
+
+	while (p > str && *p == space) {
+		*p-- = '\0';
+	}
+	return (int)(1 + p - str) * 2;
+}
+
 /** 
  * \ingroup dblib_bcp_internal
  * \brief 
