@@ -53,7 +53,7 @@
 #include <syberror.h>
 #include <dblib.h>
 
-#define TDS_FIND(k,b,c) tds_find(k, b, sizeof(b)/sizeof(b[0]), sizeof(b[0]), c)
+#define TDS_FIND(k,b,c) tds_find(k, b, TDS_VECTOR_SIZE(b), sizeof(b[0]), c)
 
 typedef bool (*compare_func)(const void *, const void *);
 
