@@ -1,4 +1,4 @@
-create table #dblib0016 (f1 int not null, f2 char(20) null, f3 binary(4) null)
+create table #dblib0016 (f1 int not null, str nvarchar(5), uuid uniqueidentifier)
 go
 select * from #dblib0016 where 0=1
 go
