@@ -28,7 +28,8 @@
 #include <sys/stat.h>
 
 #include <freetds/odbc.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
+#include <freetds/utils.h>
 #include "replacements.h"
 
 #define ODBC_PARAM(p) static const char odbc_param_##p[] = #p;
@@ -63,6 +64,9 @@ static const char *odbc_param_names[] = {
  */
 static FILE *tdoGetIniFileName(void);
 
+/* avoid name collision with system headers */
+#define SQLGetPrivateProfileString tds_SQLGetPrivateProfileString
+
 /**
  * SQLGetPrivateProfileString
  *
@@ -394,7 +398,9 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 			tds_parse_conf_section(TDS_STR_TEXTSZ, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(PacketSize)) {
 			tds_parse_conf_section(TDS_STR_BLKSZ, tds_dstr_cstr(&value), login);
-		} else if (CHK_PARAM(ClientCharset)) {
+		} else if (CHK_PARAM(ClientCharset)
+			   ||  strcasecmp(option, "client_charset") == 0) {
+			num_param = ODBC_PARAM_ClientCharset;
 			tds_parse_conf_section(TDS_STR_CLCHARSET, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(DumpFile)) {
 			tds_parse_conf_section(TDS_STR_DUMPFILE, tds_dstr_cstr(&value), login);
