@@ -25,11 +25,14 @@
 #define TDS_SDIR_SEPARATOR "\\"
 #endif
 
+#include "replacements.h"
+
 HENV odbc_env;
 HDBC odbc_conn;
 HSTMT odbc_stmt;
 int odbc_use_version3 = 0;
 void (*odbc_set_conn_attr)(void) = NULL;
+const char *odbc_conn_additional_params = NULL;
 
 char odbc_user[512];
 char odbc_server[512];
@@ -220,7 +223,7 @@ odbc_report_error(const char *errmsg, int line, const char *file)
 		else
 			fprintf(stderr, "%s\n", errmsg);
 	}
-	ret = SQLGetDiagRec(handletype, handle, 1, sqlstate, NULL, msg, ODBC_VECTOR_SIZE(msg), NULL);
+	ret = SQLGetDiagRec(handletype, handle, 1, sqlstate, NULL, msg, TDS_VECTOR_SIZE(msg), NULL);
 	if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
 		fprintf(stderr, "SQL error %s -- %s\n", C(sqlstate), C(msg));
 	odbc_disconnect();
@@ -242,7 +245,7 @@ ReportODBCError(const char *errmsg, SQLSMALLINT handletype, SQLHANDLE handle, SQ
 		else
 			fprintf(stderr, "rc=%d %s\n", (int) rc, errmsg);
 	}
-	ret = SQLGetDiagRec(handletype, handle, 1, sqlstate, NULL, msg, ODBC_VECTOR_SIZE(msg), NULL);
+	ret = SQLGetDiagRec(handletype, handle, 1, sqlstate, NULL, msg, TDS_VECTOR_SIZE(msg), NULL);
 	if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
 		fprintf(stderr, "SQL error %s -- %s\n", C(sqlstate), C(msg));
 	odbc_disconnect();
@@ -279,7 +282,19 @@ odbc_connect(void)
 	if (odbc_set_conn_attr)
 		(*odbc_set_conn_attr)();
 
-	CHKConnect(T(odbc_server), SQL_NTS, T(odbc_user), SQL_NTS, T(odbc_password), SQL_NTS, "SI");
+	if (!odbc_conn_additional_params) {
+		CHKConnect(T(odbc_server), SQL_NTS, T(odbc_user), SQL_NTS, T(odbc_password), SQL_NTS, "SI");
+	} else {
+		char *params;
+		SQLSMALLINT len;
+
+		asprintf(&params, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;%s",
+			 odbc_server, odbc_user, odbc_password, odbc_database, odbc_conn_additional_params);
+		assert(params);
+		CHKDriverConnect(NULL, T(params), SQL_NTS, (SQLTCHAR *) command, sizeof(command)/sizeof(SQLTCHAR),
+				 &len, SQL_DRIVER_NOPROMPT, "SI");
+		free(params);
+	}
 
 	CHKAllocStmt(&odbc_stmt, "S");
 
@@ -564,7 +579,7 @@ odbc_check_cursor(void)
 		SQLTCHAR output[256];
 		SQLTCHAR sqlstate[6];
 
-		CHKGetDiagRec(SQL_HANDLE_STMT, odbc_stmt, 1, sqlstate, NULL, output, ODBC_VECTOR_SIZE(output), NULL, "S");
+		CHKGetDiagRec(SQL_HANDLE_STMT, odbc_stmt, 1, sqlstate, NULL, output, TDS_VECTOR_SIZE(output), NULL, "S");
 		sqlstate[5] = 0;
 		if (strcmp(C(sqlstate), "01S02") == 0) {
 			printf("Your connection seems to not support cursors, probably you are using wrong protocol version or Sybase\n");
