@@ -466,6 +466,7 @@ AllTests(void)
 	if (odbc_db_is_microsoft() && odbc_db_version_int() >= 0x09000000u) {
 		TestInput(SQL_C_CHAR, "VARCHAR(20)", SQL_LONGVARCHAR, "VARCHAR(MAX)", "1EasyTest");
 		TestInput(SQL_C_BINARY, "VARBINARY(20)", SQL_LONGVARBINARY, "VARBINARY(MAX)", "Anything will suite!");
+		TestInput(SQL_C_CHAR, "MONEY", SQL_VARCHAR, "MONEY", "123.3456");
 	}
 	/* MSSQL 2008 */
 	if (odbc_db_is_microsoft() && odbc_db_version_int() >= 0x0a000000u) {
@@ -486,6 +487,8 @@ int
 main(int argc, char *argv[])
 {
 	odbc_use_version3 = 1;
+	odbc_conn_additional_params = "ClientCharset=ISO-8859-1;";
+
 	odbc_connect();
 
 	if (((char *) &big_endian)[0] == 1)
