@@ -38,7 +38,7 @@
 #include <ctype.h>
 
 #include "pool.h"
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <freetds/checks.h>
 #include <freetds/bytes.h>
 
@@ -65,7 +65,7 @@ dump_login(TDSLOGIN * login)
 bool
 pool_packet_read(TDSSOCKET *tds)
 {
-	int packet_len;
+	unsigned int packet_len;
 	int readed, err;
 
 	tdsdump_log(TDS_DBG_INFO1, "tds in_len %d in_pos %d\n", tds->in_len, tds->in_pos);
@@ -81,6 +81,9 @@ pool_packet_read(TDSSOCKET *tds)
 
 	/* get another packet */
 	if (tds->in_len >= packet_len) {
+		/* packet was not fully forwarded */
+		if (tds->in_pos < tds->in_len)
+			return false;
 		tds->in_pos = 0;
 		tds->in_len = 0;
 	}
@@ -92,7 +95,7 @@ pool_packet_read(TDSSOCKET *tds)
 			packet_len = TDS_GET_A2BE(&tds->in_buf[2]);
 			if (packet_len < 8)
 				break;
-			tdsdump_log(TDS_DBG_INFO1, "packet_len %d in_len %d\n", packet_len, tds->in_len);
+			tdsdump_log(TDS_DBG_INFO1, "packet_len %u in_len %d\n", packet_len, tds->in_len);
 			/* resize packet if not enough */
 			if (packet_len > tds->recv_packet->capacity) {
 				TDSPACKET *packet;
@@ -112,7 +115,7 @@ pool_packet_read(TDSSOCKET *tds)
 		assert(packet_len <= tds->recv_packet->capacity);
 		assert(tds->in_len < tds->recv_packet->capacity);
 
-		readed = read(tds_get_s(tds), &tds->in_buf[tds->in_len], packet_len - tds->in_len);
+		readed = READSOCKET(tds_get_s(tds), &tds->in_buf[tds->in_len], packet_len - tds->in_len);
 		tdsdump_log(TDS_DBG_INFO1, "readed %d\n", readed);
 
 		/* socket closed */
