@@ -4,9 +4,7 @@
  * public domain.  no warranty.  use at your own risk.  have a nice day.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -15,13 +13,11 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds_sysdep_private.h"
+#include <freetds/sysdep_private.h>
 #include "replacements.h"
 
-TDS_RCSID(var, "$Id: asprintf.c,v 1.7 2006/08/23 14:26:20 freddy77 Exp $");
-
 int
-asprintf(char **ret, const char *fmt, ...)
+tds_asprintf(char **ret, const char *fmt, ...)
 {
 	int len;
 	va_list ap;
