@@ -17,21 +17,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds_sysdep_private.h"
+#include <freetds/sysdep_private.h>
 #include "replacements.h"
 
 #if ! HAVE_BASENAME
 
-TDS_RCSID(var, "$Id: basename.c,v 1.5 2010/01/10 14:43:11 freddy77 Exp $");
-
 #ifdef _WIN32
 #define TDS_ISDIR_SEPARATOR(c) ((c) == '/' || (c) == '\\')
 #else
