@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998-1999  Brian Bruns
- * Copyright (C) 2005-2011  Frediano Ziglio
+ * Copyright (C) 2005-2015  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -36,24 +34,17 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds.h"
-#include "tdsbytes.h"
-#include "tdsstring.h"
-#include "md4.h"
-#include "md5.h"
-#include "hmac_md5.h"
-#include "des.h"
-#include "tdsiconv.h"
-
-#if defined(HAVE_OPENSSL)
-#include <openssl/rand.h>
-#endif
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: challenge.c,v 1.47.2.1 2011/04/11 13:40:45 freddy77 Exp $");
+#include <freetds/time.h>
+#include <freetds/tds.h>
+#include <freetds/bytes.h>
+#include <freetds/utils/string.h>
+#include <freetds/iconv.h>
+#include <freetds/utils.h>
+#include <freetds/utils/md4.h>
+#include <freetds/utils/md5.h>
+#include <freetds/utils/hmac_md5.h>
+#include <freetds/utils/des.h>
+#include "replacements.h"
 
 /**
  * \ingroup libtds
@@ -70,11 +61,6 @@ TDS_RCSID(var, "$Id: challenge.c,v 1.47.2.1 2011/04/11 13:40:45 freddy77 Exp $")
  * The following code is based on some psuedo-C code from ronald@innovation.ch
  */
 
-#ifndef offsetof
-# define offsetof(st, m) \
-     ((size_t) ( (char *)&((st *)(0))->m - (char *)0 ))
-#endif
-
 typedef struct tds_answer
 {
 	unsigned char lm_resp[24];
@@ -84,23 +70,23 @@ typedef struct tds_answer
 
 typedef struct
 {
-	TDS_TINYINT     response_type;
-	TDS_TINYINT     max_response_type;
-	TDS_SMALLINT    reserved1;
-	TDS_UINT        reserved2;
-	TDS_UINT8       timestamp;
-	TDS_UCHAR       challenge[8];
-	TDS_UINT        unknown;
+	uint8_t  response_type;
+	uint8_t  max_response_type;
+	uint16_t reserved1;
+	uint32_t reserved2;
+	uint64_t timestamp;
+	uint8_t  challenge[8];
+	uint32_t unknown;
 	/* target info block - variable length */
-	TDS_UCHAR	target_info[4];
+	uint8_t  target_info[4];
 } names_blob_prefix_t;
 
-static int
+static TDSRET
 tds_answer_challenge(TDSSOCKET * tds,
-		     TDSCONNECTION * connection,
+		     TDSLOGIN * login,
 		     const unsigned char *challenge,
-		     TDS_UINT * flags,
-		     const unsigned char *names_blob, TDS_INT names_blob_len, TDSANSWER * answer, unsigned char **ntlm_v2_response);
+		     uint32_t * flags,
+		     const unsigned char *names_blob, int names_blob_len, TDSANSWER * answer, unsigned char **ntlm_v2_response);
 static void tds_encrypt_answer(const unsigned char *hash, const unsigned char *challenge, unsigned char *answer);
 static void tds_convert_key(const unsigned char *key_56, DES_KEY * ks);
 
@@ -120,7 +106,7 @@ convert_to_usc2le_string(TDSSOCKET * tds, const char *s, size_t len, char *out)
 	char *ob;
 	size_t il, ol;
 
-	const TDSICONV * char_conv = tds->char_convs[client2ucs2];
+	TDSICONV * char_conv = tds->conn->char_convs[client2ucs2];
 
 	/* char_conv is only mostly const */
 	TDS_ERRNO_MESSAGE_FLAGS *suppress = (TDS_ERRNO_MESSAGE_FLAGS *) & char_conv->suppress;
@@ -142,25 +128,7 @@ convert_to_usc2le_string(TDSSOCKET * tds, const char *s, size_t len, char *out)
 	return ob - out;
 }
 
-
-static void
-generate_random_buffer(unsigned char *out, int len)
-{
-	int i;
-
-#if defined(HAVE_OPENSSL)
-	if (RAND_bytes(out, len) == 1)
-		return;
-	if (RAND_pseudo_bytes(out, len) >= 0)
-		return;
-#endif
-
-	/* TODO find a better random... */
-	for (i = 0; i < len; ++i)
-		out[i] = rand() / (RAND_MAX / 256);
-}
-
-static int
+static TDSRET
 make_ntlm_hash(TDSSOCKET * tds, const char *passwd, unsigned char ntlm_hash[16])
 {
 	MD4_CTX context;
@@ -187,11 +155,11 @@ make_ntlm_hash(TDSSOCKET * tds, const char *passwd, unsigned char ntlm_hash[16])
 	/* with security is best be pedantic */
 	memset((char *) passwd_usc2le, 0, passwd_usc2le_len);
 	memset(&context, 0, sizeof(context));
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 
-static int
+static TDSRET
 make_ntlm_v2_hash(TDSSOCKET * tds, const char *passwd, unsigned char ntlm_v2_hash[16])
 {
 	const char *user_name, *domain;
@@ -201,10 +169,9 @@ make_ntlm_v2_hash(TDSSOCKET * tds, const char *passwd, unsigned char ntlm_v2_has
 	unsigned char ntlm_hash[16];
 	char buf[128];
 	char buf_usc2le[512];
-	int res;
+	TDSRET res;
 
-	user_name = tds_dstr_cstr(&tds->connection->user_name);
-	user_name_len = strlen(user_name);
+	user_name = tds_dstr_cstr(&tds->login->user_name);
 
 	/* parse domain\username */
 	p = strchr(user_name, '\\');
@@ -253,12 +220,12 @@ make_ntlm_v2_hash(TDSSOCKET * tds, const char *passwd, unsigned char ntlm_v2_has
  */
 static unsigned char *
 make_lm_v2_response(const unsigned char ntlm_v2_hash[16],
-		    const unsigned char *client_data, TDS_INT client_data_len, const unsigned char challenge[8])
+		    const unsigned char *client_data, int client_data_len, const unsigned char challenge[8])
 {
 	int mac_len = 16 + client_data_len;
 	unsigned char *mac;
 
-	mac = malloc(mac_len);
+	mac = tds_new(unsigned char, mac_len);
 	if (!mac)
 		return NULL;
 
@@ -269,23 +236,26 @@ make_lm_v2_response(const unsigned char ntlm_v2_hash[16],
 	return mac;
 }
 
-static int
+static TDSRET
 tds_answer_challenge_ntlmv2(TDSSOCKET * tds,
-		     TDSCONNECTION * connection,
+		     TDSLOGIN * login,
 		     const unsigned char *challenge,
-		     TDS_UINT * flags,
-		     const unsigned char *names_blob, TDS_INT names_blob_len, TDSANSWER * answer, unsigned char **ntlm_v2_response)
+		     uint32_t * flags,
+		     const unsigned char *names_blob, int names_blob_len, TDSANSWER * answer, unsigned char **ntlm_v2_response)
 {
-	int res;
-	const char *passwd = tds_dstr_cstr(&connection->password);
+	TDSRET res;
+	const char *passwd = tds_dstr_cstr(&login->password);
 
 	/* NTLMv2 */
 	unsigned char *lm_v2_response;
 	unsigned char ntlm_v2_hash[16];
 	const names_blob_prefix_t *names_blob_prefix;
 
+	if (!names_blob)
+		return TDS_FAIL;
+
 	res = make_ntlm_v2_hash(tds, passwd, ntlm_v2_hash);
-	if (res != TDS_SUCCEED)
+	if (TDS_FAILED(res))
 		return res;
 
 	/* LMv2 response */
@@ -307,7 +277,7 @@ tds_answer_challenge_ntlmv2(TDSSOCKET * tds,
 
 	/* local not supported */
 	*flags &= ~0x4000;
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 /**
@@ -317,29 +287,29 @@ tds_answer_challenge_ntlmv2(TDSSOCKET * tds,
  * @param flags NTLM flags from server side
  * @param answer buffer where to store crypted password
  */
-static int
+static TDSRET
 tds_answer_challenge(TDSSOCKET * tds,
-		     TDSCONNECTION * connection,
+		     TDSLOGIN * login,
 		     const unsigned char *challenge,
-		     TDS_UINT * flags,
-		     const unsigned char *names_blob, TDS_INT names_blob_len, TDSANSWER * answer, unsigned char **ntlm_v2_response)
+		     uint32_t * flags,
+		     const unsigned char *names_blob, int names_blob_len, TDSANSWER * answer, unsigned char **ntlm_v2_response)
 {
 #define MAX_PW_SZ 14
-	const char *passwd = tds_dstr_cstr(&connection->password);
+	const char *passwd = tds_dstr_cstr(&login->password);
 	DES_KEY ks;
 	unsigned char hash[24], ntlm2_challenge[16];
-	int res;
+	TDSRET res;
 
 	memset(answer, 0, sizeof(TDSANSWER));
 
-	if (connection->use_ntlmv2) {
-		return tds_answer_challenge_ntlmv2(tds, connection, challenge, flags,
+	if (login->use_ntlmv2) {
+		return tds_answer_challenge_ntlmv2(tds, login, challenge, flags,
 						   names_blob, names_blob_len, answer, ntlm_v2_response);
 	} else if ((*flags & 0x80000) != 0) {
 		/* NTLM2 */
 		MD5_CTX md5_ctx;
 
-		generate_random_buffer(hash, 8);
+		tds_random_buffer(hash, 8);
 		memset(hash + 8, 0, 16);
 		memcpy(answer->lm_resp, hash, 24);
 
@@ -349,9 +319,8 @@ tds_answer_challenge(TDSSOCKET * tds,
 		MD5Final(&md5_ctx, ntlm2_challenge);
 		challenge = ntlm2_challenge;
 		memset(&md5_ctx, 0, sizeof(md5_ctx));
-	} else {
+	} else if (login->use_lanman) {
 		/* LM */
-#if TDS_USE_LM
 		size_t len, i;
 		unsigned char passwd_buf[MAX_PW_SZ];
 		static const des_cblock magic = { 0x4B, 0x47, 0x53, 0x21, 0x40, 0x23, 0x24, 0x25 };
@@ -376,9 +345,8 @@ tds_answer_challenge(TDSSOCKET * tds,
 
 		tds_encrypt_answer(hash, challenge, answer->lm_resp);
 		memset(passwd_buf, 0, sizeof(passwd_buf));
-#else
+	} else {
 		memset(answer->lm_resp, 0, sizeof(answer->lm_resp));
-#endif
 	}
 	*flags = 0x8201;
 
@@ -443,9 +411,9 @@ tds_convert_key(const unsigned char *key_56, DES_KEY * ks)
 	memset(&key, 0, sizeof(key));
 }
 
-static int
+static TDSRET
 tds7_send_auth(TDSSOCKET * tds,
-	       const unsigned char *challenge, TDS_UINT flags, const unsigned char *names_blob, TDS_INT names_blob_len)
+	       const unsigned char *challenge, uint32_t flags, const unsigned char *names_blob, int names_blob_len)
 {
 	size_t current_pos;
 	TDSANSWER answer;
@@ -453,39 +421,67 @@ tds7_send_auth(TDSSOCKET * tds,
 	/* FIXME: stuff duplicate in tds7_send_login */
 	const char *domain;
 	const char *user_name;
+	const char *host_name;
 	const char *p;
-	size_t user_name_len, host_name_len, password_len, domain_len;
-	int rc;
+	char *convert_buffer;
+	size_t user_name_len, host_name_len, domain_len;
+	TDSRET rc;
 
 	unsigned char *ntlm_v2_response = NULL;
 	unsigned int ntlm_response_len = 24;
 	const unsigned int lm_response_len = 24;
 
-	TDSCONNECTION *connection = tds->connection;
+	TDSLOGIN *login = tds->login;
 
 	/* check connection */
-	if (!connection)
+	if (!login)
 		return TDS_FAIL;
 
 	/* parse a bit of config */
-	user_name = tds_dstr_cstr(&connection->user_name);
-	user_name_len = user_name ? strlen(user_name) : 0;
-	host_name_len = tds_dstr_len(&connection->client_host_name);
-	password_len = tds_dstr_len(&connection->password);
+	user_name = tds_dstr_cstr(&login->user_name);
+	user_name_len = tds_dstr_len(&login->user_name);
+	host_name = tds_dstr_cstr(&login->client_host_name);
+	host_name_len = tds_dstr_len(&login->client_host_name);
+
+	/* convert strings */
+	convert_buffer = malloc((user_name_len + host_name_len) * 2);
+	if (!convert_buffer)
+		return TDS_FAIL;
+
+	user_name_len = convert_to_usc2le_string(tds, user_name, user_name_len, convert_buffer);
+	user_name = convert_buffer;
+	if (user_name_len != (size_t) -1) {
+		host_name_len = convert_to_usc2le_string(tds, host_name, host_name_len, convert_buffer + user_name_len);
+		host_name = convert_buffer + user_name_len;
+	}
+	if (user_name_len == (size_t) -1 || host_name_len == (size_t) -1) {
+		free(convert_buffer);
+		return TDS_FAIL;
+	}
 
 	/* parse domain\username */
-	if ((p = strchr(user_name, '\\')) == NULL)
-		return TDS_FAIL;
+	p = user_name;
+	for (;;) {
+		if (p >= user_name + user_name_len) {
+			free(convert_buffer);
+			return TDS_FAIL;
+		}
+		if (p[0] == '\\' && p[1] == 0)
+			break;
+		p += 2;
+	}
 
 	domain = user_name;
 	domain_len = p - user_name;
 
-	user_name = p + 1;
-	user_name_len = strlen(user_name);
+	user_name = p + 2;
+	user_name_len = domain + user_name_len - user_name;
 
-	rc = tds_answer_challenge(tds, connection, challenge, &flags, names_blob, names_blob_len, &answer, &ntlm_v2_response);
-	if (rc != TDS_SUCCEED)
+	rc = tds_answer_challenge(tds, login, challenge, &flags, names_blob, names_blob_len, &answer, &ntlm_v2_response);
+	if (TDS_FAILED(rc)) {
+		free(convert_buffer);
 		return rc;
+	}
 
 	ntlm_response_len = ntlm_v2_response ? 16 + names_blob_len : 24;
 	/* ntlm_response_len = 0; */
@@ -494,8 +490,7 @@ tds7_send_auth(TDSSOCKET * tds,
 	tds_put_n(tds, "NTLMSSP", 8);
 	tds_put_int(tds, 3);	/* sequence 3 */
 
-	/* FIXME *2 work only for single byte encodings */
-	current_pos = 64u + (domain_len + user_name_len + host_name_len) * 2u;
+	current_pos = 64u + domain_len + user_name_len + host_name_len;
 
 	/* LM/LMv2 Response */
 	tds_put_smallint(tds, lm_response_len);	/* lan man resp length */
@@ -511,22 +506,22 @@ tds7_send_auth(TDSSOCKET * tds,
 	current_pos = 64;
 
 	/* Target Name - domain or server name */
-	TDS_PUT_SMALLINT(tds, domain_len * 2);
-	TDS_PUT_SMALLINT(tds, domain_len * 2);
+	TDS_PUT_SMALLINT(tds, domain_len);
+	TDS_PUT_SMALLINT(tds, domain_len);
 	TDS_PUT_INT(tds, current_pos);
-	current_pos += domain_len * 2u;
+	current_pos += domain_len;
 
 	/* username */
-	TDS_PUT_SMALLINT(tds, user_name_len * 2);
-	TDS_PUT_SMALLINT(tds, user_name_len * 2);
+	TDS_PUT_SMALLINT(tds, user_name_len);
+	TDS_PUT_SMALLINT(tds, user_name_len);
 	TDS_PUT_INT(tds, current_pos);
-	current_pos += user_name_len * 2u;
+	current_pos += user_name_len;
 
 	/* Workstation Name */
-	TDS_PUT_SMALLINT(tds, host_name_len * 2);
-	TDS_PUT_SMALLINT(tds, host_name_len * 2);
+	TDS_PUT_SMALLINT(tds, host_name_len);
+	TDS_PUT_SMALLINT(tds, host_name_len);
 	TDS_PUT_INT(tds, current_pos);
-	current_pos += host_name_len * 2u;
+	current_pos += host_name_len;
 
 	/* Session Key (optional) */
 	tds_put_smallint(tds, 0);
@@ -535,15 +530,15 @@ tds7_send_auth(TDSSOCKET * tds,
 
 	/* flags */
 	/* "challenge" is 8 bytes long */
-	/* tds_answer_challenge(tds_dstr_cstr(&connection->password), challenge, &flags, &answer); */
+	/* tds_answer_challenge(tds_dstr_cstr(&login->password), challenge, &flags, &answer); */
 	tds_put_int(tds, flags);
 
 	/* OS Version Structure (Optional) */
 
 	/* Data itself */
-	tds_put_string(tds, domain, (int)domain_len);
-	tds_put_string(tds, user_name, (int)user_name_len);
-	tds_put_string(tds, tds_dstr_cstr(&connection->client_host_name), (int)host_name_len);
+	tds_put_n(tds, domain, domain_len);
+	tds_put_n(tds, user_name, user_name_len);
+	tds_put_n(tds, host_name, host_name_len);
 
 	/* data block */
 	tds_put_n(tds, answer.lm_resp, lm_response_len);
@@ -561,6 +556,7 @@ tds7_send_auth(TDSSOCKET * tds,
 	/* for security reason clear structure */
 	memset(&answer, 0, sizeof(TDSANSWER));
 
+	free(convert_buffer);
 	return tds_flush_packet(tds);
 }
 
@@ -569,15 +565,15 @@ typedef struct tds_ntlm_auth
 	TDSAUTHENTICATION tds_auth;
 } TDSNTLMAUTH;
 
-static int
-tds_ntlm_free(TDSSOCKET * tds, TDSAUTHENTICATION * tds_auth)
+static TDSRET
+tds_ntlm_free(TDSCONNECTION * conn, TDSAUTHENTICATION * tds_auth)
 {
 	TDSNTLMAUTH *auth = (TDSNTLMAUTH *) tds_auth;
 
 	free(auth->tds_auth.packet);
 	free(auth);
 
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 static const unsigned char ntlm_id[] = "NTLMSSP";
@@ -587,24 +583,17 @@ static const unsigned char ntlm_id[] = "NTLMSSP";
  * This takes GMT as input
  */
 static void
-unix_to_nt_time(TDS_UINT8 * nt, time_t t)
+unix_to_nt_time(uint64_t * nt, struct timeval *tv)
 {
-#define TIME_FIXUP_CONSTANT (((TDS_UINT8) 134774U) * 86400U)
+	/* C time start on 1970, nt time on 1600 */
+#define TIME_FIXUP_CONSTANT (((uint64_t) 134774U) * 86400U)
 
-	TDS_UINT8 t2;
+	uint64_t t2;
 
-	if (t == (time_t) - 1) {
-		*nt = (TDS_UINT8) - ((TDS_INT8) 1);
-		return;
-	}
-	if (t == 0) {
-		*nt = 0;
-		return;
-	}
-
-	t2 = t;
+	t2 = tv->tv_sec;
 	t2 += TIME_FIXUP_CONSTANT;
-	t2 *= 1000 * 1000 * 10;
+	t2 *= 1000u * 1000u * 10u;
+	t2 += tv->tv_usec * 10u;
 
 	*nt = t2;
 }
@@ -612,42 +601,39 @@ unix_to_nt_time(TDS_UINT8 * nt, time_t t)
 static void
 fill_names_blob_prefix(names_blob_prefix_t * prefix)
 {
-	TDS_UINT8 nttime = 0;
+	struct timeval tv;
+	uint64_t nttime = 0;
 
-	/* TODO use more precision, not only seconds */
-	unix_to_nt_time(&nttime, time(NULL));
+	gettimeofday(&tv, NULL);
+	unix_to_nt_time(&nttime, &tv);
 
 	prefix->response_type = 0x01;
 	prefix->max_response_type = 0x01;
 	prefix->reserved1 = 0x0000;
 	prefix->reserved2 = 0x00000000;
 #ifdef WORDS_BIGENDIAN
-	tds_swap_bytes((unsigned char *) &nttime, 8);
+	tds_swap_bytes(&nttime, 8);
 #endif
 	prefix->timestamp = nttime;
-	generate_random_buffer(prefix->challenge, sizeof(prefix->challenge));
+	tds_random_buffer(prefix->challenge, sizeof(prefix->challenge));
 
 	prefix->unknown = 0x00000000;
 }
 
-static int
+static TDSRET
 tds_ntlm_handle_next(TDSSOCKET * tds, struct tds_authentication * auth, size_t len)
 {
 	const int length = (int)len;
 	unsigned char nonce[8];
-	TDS_UINT flags;
+	uint32_t flags;
 	int where;
 
-	int domain_len;
 	int data_block_offset;
 
-	int target_info_len = 0;
-	int target_info_offset;
+	int names_blob_len = 0;
+	unsigned char *names_blob = NULL;
 
-	int names_blob_len;
-	unsigned char *names_blob;
-
-	int rc;
+	TDSRET rc;
 
 	/* at least 32 bytes (till context) */
 	if (len < 32)
@@ -658,8 +644,8 @@ tds_ntlm_handle_next(TDSSOCKET * tds, struct tds_authentication * auth, size_t l
 		return TDS_FAIL;
 	if (tds_get_int(tds) != 2)	/* sequence -> 2 */
 		return TDS_FAIL;
-	domain_len = tds_get_smallint(tds);	/* domain len */
-	domain_len = tds_get_smallint(tds);	/* domain len */
+	tds_get_smallint(tds);	/* domain len */
+	tds_get_smallint(tds);	/* domain len */
 	data_block_offset = tds_get_int(tds);	/* domain offset */
 	flags = tds_get_int(tds);	/* flags */
 	tds_get_n(tds, nonce, 8);
@@ -670,6 +656,8 @@ tds_ntlm_handle_next(TDSSOCKET * tds, struct tds_authentication * auth, size_t l
 	/* Version 1 -- The Context, Target Information, and OS Version structure are all omitted */
 
 	if (data_block_offset >= 48 && where + 16 <= length) {
+		int target_info_len, target_info_offset;
+
 		/* Version 2 -- The Context and Target Information fields are present, but the OS Version structure is not. */
 		tds_get_n(tds, NULL, 8);	/* Context (two consecutive longs) */
 
@@ -689,32 +677,29 @@ tds_ntlm_handle_next(TDSSOCKET * tds, struct tds_authentication * auth, size_t l
 #endif
 			where += 8;
 		}
-	}
 
-	/* read Target Info if possible */
-	if (target_info_len > 0 && target_info_offset >= where && target_info_offset + target_info_len <= length) {
-		tds_get_n(tds, NULL, target_info_offset - where);
-		where = target_info_offset;
+		/* read Target Info if possible */
+		if (target_info_len > 0 && target_info_offset >= where && target_info_offset + target_info_len <= length) {
+			tds_get_n(tds, NULL, target_info_offset - where);
+			where = target_info_offset;
 
-		/*
-		 * the + 4 came from blob structure, after Target Info 4
-		 * additional reserved bytes must be present
-		 * Search "davenport port"
-		 * (currently http://davenport.sourceforge.net/ntlm.html)
-		 */
-		names_blob_len = offsetof(names_blob_prefix_t, target_info) + target_info_len + 4;
+			/*
+			 * the + 4 came from blob structure, after Target Info 4
+			 * additional reserved bytes must be present
+			 * Search "davenport port"
+			 * (currently http://davenport.sourceforge.net/ntlm.html)
+			 */
+			names_blob_len = TDS_OFFSET(names_blob_prefix_t, target_info) + target_info_len + 4;
 
-		/* read Target Info */
-		names_blob = (unsigned char *) calloc(names_blob_len, 1);
-		if (!names_blob)
-			return TDS_FAIL;
+			/* read Target Info */
+			names_blob = tds_new0(unsigned char, names_blob_len);
+			if (!names_blob)
+				return TDS_FAIL;
 
-		fill_names_blob_prefix((names_blob_prefix_t *) names_blob);
-		tds_get_n(tds, names_blob + offsetof(names_blob_prefix_t, target_info), target_info_len);
-		where += target_info_len;
-	} else {
-		names_blob = NULL;
-		names_blob_len = 0;
+			fill_names_blob_prefix((names_blob_prefix_t *) names_blob);
+			tds_get_n(tds, names_blob + TDS_OFFSET(names_blob_prefix_t, target_info), target_info_len);
+			where += target_info_len;
+		}
 	}
 	/* discard anything left */
 	tds_get_n(tds, NULL, length - where);
@@ -738,17 +723,17 @@ tds_ntlm_get_auth(TDSSOCKET * tds)
 	const char *domain;
 	const char *user_name;
 	const char *p;
-	TDS_UCHAR *packet;
+	uint8_t *packet;
 	int host_name_len;
 	int domain_len;
 	int auth_len;
 	struct tds_ntlm_auth *auth;
 
-	if (!tds->connection)
+	if (!tds->login)
 		return NULL;
 
-	user_name = tds_dstr_cstr(&tds->connection->user_name);
-	host_name_len = (int)tds_dstr_len(&tds->connection->client_host_name);
+	user_name = tds_dstr_cstr(&tds->login->user_name);
+	host_name_len = (int)tds_dstr_len(&tds->login->client_host_name);
 
 	/* check override of domain */
 	if ((p = strchr(user_name, '\\')) == NULL)
@@ -757,7 +742,7 @@ tds_ntlm_get_auth(TDSSOCKET * tds)
 	domain = user_name;
 	domain_len = (int)(p - user_name);
 
-	auth = (struct tds_ntlm_auth *) calloc(1, sizeof(struct tds_ntlm_auth));
+	auth = tds_new0(struct tds_ntlm_auth, 1);
 
 	if (!auth)
 		return NULL;
@@ -766,7 +751,7 @@ tds_ntlm_get_auth(TDSSOCKET * tds)
 	auth->tds_auth.handle_next = tds_ntlm_handle_next;
 
 	auth->tds_auth.packet_len = auth_len = 40 + host_name_len + domain_len;
-	auth->tds_auth.packet = packet = malloc(auth_len);
+	auth->tds_auth.packet = packet = tds_new(uint8_t, auth_len);
 	if (!packet) {
 		free(auth);
 		return NULL;
@@ -798,7 +783,7 @@ tds_ntlm_get_auth(TDSSOCKET * tds)
 	TDS_PUT_A4(packet + 36, TDS_HOST4LE(0x0f000000));
 
 	/* hostname and domain */
-	memcpy(packet + 40, tds_dstr_cstr(&tds->connection->client_host_name), host_name_len);
+	memcpy(packet + 40, tds_dstr_cstr(&tds->login->client_host_name), host_name_len);
 	memcpy(packet + 40 + host_name_len, domain, domain_len);
 
 	return (TDSAUTHENTICATION *) auth;
