@@ -39,6 +39,7 @@
 #include <freetds/tds.h>
 #include <freetds/iconv.h>
 #include <freetds/bool.h>
+#include <freetds/bytes.h>
 #if HAVE_ICONV
 #include <iconv.h>
 #endif
@@ -47,11 +48,10 @@
 				(charset)->min_bytes_per_char : 0 )
 
 
-static int collate2charset(int sql_collate, int lcid);
+static int collate2charset(TDSCONNECTION * conn, TDS_UCHAR collate[5]);
 static size_t skip_one_input_sequence(iconv_t cd, const TDS_ENCODING * charset, const char **input, size_t * input_size);
 static int tds_iconv_info_init(TDSICONV * char_conv, int client_canonic, int server_canonic);
 static int tds_iconv_init(void);
-static int tds_canonical_charset(const char *charset_name);
 static void _iconv_close(iconv_t * cd);
 static void tds_iconv_info_close(TDSICONV * char_conv);
 
@@ -63,16 +63,26 @@ static void tds_iconv_info_close(TDSICONV * char_conv);
  */
 
 #define TDS_ICONV_ENCODING_TABLES
-#include "encodings.h"
+#include <freetds/encodings.h>
 
 /* this will contain real iconv names */
-static const char *iconv_names[sizeof(canonic_charsets) / sizeof(canonic_charsets[0])];
+static const char *iconv_names[TDS_VECTOR_SIZE(canonic_charsets)];
 static int iconv_initialized = 0;
 static const char *ucs2name;
 
 enum
 { POS_ISO1, POS_UTF8, POS_UCS2LE, POS_UCS2BE };
 
+static const struct {
+	uint8_t len;
+	char data[15];
+} test_strings[4] = {
+	{ 4, "Ao\xD3\xE5" },
+	{ 6, "Ao\xC3\x93\xC3\xA5" },
+	{ 8, "A\x00o\x000\xD3\x00\xE5\x00" },
+	{ 8, "\x00" "A\x00o\x000\xD3\x00\xE5" },
+};
+
 /**
  * Initialize charset searching for UTF-8, UCS-2 and ISO8859-1
  */
@@ -186,6 +196,35 @@ tds_iconv_init(void)
 		tdsdump_log(TDS_DBG_INFO1, "local name for %s is %s\n", canonic_charsets[i].name,
 			    iconv_names[i] ? iconv_names[i] : "(null)");
 
+	/* base conversions checks */
+	for (i = 0; i < 4 * 4; ++i) {
+		const int from = i / 4;
+		const int to = i % 4;
+		char ob[16];
+		size_t il, ol;
+		ICONV_CONST char *pib;
+		char *pob;
+		size_t res;
+
+		if (!iconv_names[from] || !iconv_names[to])
+			continue;
+		cd = tds_sys_iconv_open(iconv_names[to], iconv_names[from]);
+		if (cd == (iconv_t) -1)
+			return 1;
+
+		pib = (ICONV_CONST char *) test_strings[from].data;
+		il = test_strings[from].len;
+		pob = ob;
+		ol = sizeof(ob);
+		res = tds_sys_iconv(cd, &pib, &il, &pob, &ol);
+		tds_sys_iconv_close(cd);
+
+		if (res != 0
+		    || sizeof(ob) - ol != test_strings[to].len
+		    || memcmp(ob, test_strings[to].data, test_strings[to].len) != 0)
+			return 1;
+	}
+
 	/* success (it should always occurs) */
 	return 0;
 }
@@ -332,7 +371,7 @@ tds_iconv_open(TDSCONNECTION * conn, const char *charset, int use_utf16)
 	/* initialize */
 	if (!iconv_initialized) {
 		if ((ret = tds_iconv_init()) > 0) {
-			static const char names[][12] = { "ISO 8859-1", "UTF-8" };
+			static const char names[][12] = { "ISO 8859-1", "UCS-2" };
 			assert(ret < 3);
 			tdsdump_log(TDS_DBG_FUNC, "error: tds_iconv_init() returned %d; "
 						  "could not find a name for %s that your iconv accepts.\n"
@@ -597,12 +636,13 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		size_t len = *inbytesleft < *outbytesleft ? *inbytesleft : *outbytesleft;
 
 		memcpy(*outbuf, *inbuf, len);
-		errno = *inbytesleft > *outbytesleft ? E2BIG : 0;
+		conv_errno = *inbytesleft > *outbytesleft ? E2BIG : 0;
 		*inbytesleft -= len;
 		*outbytesleft -= len;
 		*inbuf += len;
 		*outbuf += len;
-		return 0;
+		errno = conv_errno;
+		return conv_errno ? (size_t) -1 : 0;
 	}
 
 	/*
@@ -712,7 +752,7 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 /**
  * Get a iconv info structure, allocate and initialize if needed
  */
-static TDSICONV *
+TDSICONV *
 tds_iconv_get_info(TDSCONNECTION * conn, int canonic_client, int canonic_server)
 {
 	TDSICONV *info;
@@ -809,9 +849,9 @@ tds_srv_charset_changed(TDSCONNECTION * conn, const char *charset)
 
 /* change singlebyte conversions according to server */
 void
-tds7_srv_charset_changed(TDSCONNECTION * conn, int sql_collate, int lcid)
+tds7_srv_charset_changed(TDSCONNECTION * conn, TDS_UCHAR collation[5])
 {
-	tds_srv_charset_changed_num(conn, collate2charset(sql_collate, lcid));
+	tds_srv_charset_changed_num(conn, collate2charset(conn, collation));
 }
 
 /**
@@ -917,36 +957,18 @@ skip_charsize:
 	return charsize;
 }
 
-static int
-lookup_canonic(const CHARACTER_SET_ALIAS aliases[], const char *charset_name)
-{
-	int i;
-
-	for (i = 0; aliases[i].alias; ++i) {
-		if (0 == strcmp(charset_name, aliases[i].alias))
-			return aliases[i].canonic;
-	}
-
-	return -1;
-}
+#include <freetds/charset_lookup.h>
 
 /**
  * Determine canonical iconv character set.
  * \returns canonical position, or -1 if lookup failed.
  * \remarks Returned name can be used in bytes_per_char(), above.
  */
-static int
+int
 tds_canonical_charset(const char *charset_name)
 {
-	int res;
-
-	/* search in alternative */
-	res = lookup_canonic(iconv_aliases, charset_name);
-	if (res >= 0)
-		return res;
-
-	/* search in sybase */
-	return lookup_canonic(sybase_aliases, charset_name);
+	const struct charset_alias *c = charset_lookup(charset_name, strlen(charset_name));
+	return c ? c->canonic : -1;
 }
 
 /**
@@ -968,15 +990,17 @@ tds_canonical_charset_name(const char *charset_name)
 }
 
 static int
-collate2charset(int sql_collate, int lcid)
+collate2charset(TDSCONNECTION * conn, TDS_UCHAR collate[5])
 {
+	int cp = 0;
+	const int sql_collate = collate[4];
+	const int lcid = TDS_GET_UA2LE(collate);
+
 	/*
 	 * The table from the MSQLServer reference "Windows Collation Designators" 
 	 * and from " NLS Information for Microsoft Windows XP"
 	 */
 
-	int cp = 0;
-
 	switch (sql_collate) {
 	case 30:		/* SQL_Latin1_General_CP437_BIN */
 	case 31:		/* SQL_Latin1_General_CP437_CS_AS */
@@ -1023,7 +1047,7 @@ collate2charset(int sql_collate, int lcid)
 		return TDS_CHARSET_CP1257;
 	}
 
-	switch (lcid & 0xffff) {
+	switch (lcid) {
 	case 0x405:
 	case 0x40e:		/* 0x1040e */
 	case 0x415:
@@ -1209,9 +1233,7 @@ collate2charset(int sql_collate, int lcid)
 TDSICONV *
 tds_iconv_from_collate(TDSCONNECTION * conn, TDS_UCHAR collate[5])
 {
-	const int sql_collate = collate[4];
-	const int lcid = collate[1] * 256 + collate[0];
-	int canonic_charset = collate2charset(sql_collate, lcid);
+	int canonic_charset = collate2charset(conn, collate);
 
 	/* same as client (usually this is true, so this improve performance) ? */
 	if (conn->char_convs[client2server_chardata]->to.charset.canonic == canonic_charset)
