@@ -413,6 +413,8 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 	struct addrinfo *addrs;
 	int orig_port;
 	bool rerouted = false;
+	/* save to restore during redirected connection */
+	unsigned int orig_mars = login->mars;
 
 	/*
 	 * A major version of 0 means try to guess the TDS version. 
@@ -486,19 +488,6 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 	tds->login = login;
 
 	tds->conn->tds_version = login->tds_version;
-	tds->conn->emul_little_endian = login->emul_little_endian;
-#ifdef WORDS_BIGENDIAN
-	/*
-	 * Enable automatically little endian emulation.
-	 * TDS 7/8 only supports little endian.
-	 * This is done even for 4.2 to ensure that when we connect to a
-	 * MSSQL we use it and avoid to make additional checks for
-	 * broken 7.0 servers.
-	 * Note that 4.2 should not be used anymore for real jobs.
-	 */
-	if (IS_TDS7_PLUS(tds->conn) || IS_TDS42(tds->conn))
-		tds->conn->emul_little_endian = 1;
-#endif
 
 	/* set up iconv if not already initialized*/
 	if (tds->conn->char_convs[client2ucs2]->to.cd == (iconv_t) -1) {
@@ -586,11 +575,27 @@ reroute:
 	}
 
 	/* need to do rerouting */
-	if (IS_TDS71_PLUS(tds->conn) && !rerouted
+	if (IS_TDS71_PLUS(tds->conn)
 	    && !tds_dstr_isempty(&login->routing_address) && login->routing_port) {
 		TDSRET ret;
+		char *server_name = NULL;
 
 		tds_close_socket(tds);
+		/* only one redirection is allowed */
+		if (rerouted) {
+			tdserror(tds_get_ctx(tds), tds, TDSEFCON, 0);
+			return -TDSEFCON;
+		}
+		if (asprintf(&server_name, "%s,%d", tds_dstr_cstr(&login->routing_address), login->routing_port) < 0) {
+			tdserror(tds_get_ctx(tds), tds, TDSEFCON, 0);
+			return -TDSEMEM;
+		}
+		if (!tds_dstr_set(&login->server_name, server_name)) {
+			free(server_name);
+			tdserror(tds_get_ctx(tds), tds, TDSEFCON, 0);
+			return -TDSEMEM;
+		}
+		login->mars = orig_mars;
 		login->port = login->routing_port;
 		ret = tds_lookup_host_set(tds_dstr_cstr(&login->routing_address), &login->ip_addrs);
 		login->routing_port = 0;
@@ -606,9 +611,28 @@ reroute:
 #if ENABLE_ODBC_MARS
 	/* initialize SID */
 	if (IS_TDS72_PLUS(tds->conn) && login->mars) {
-		tds->conn->sessions[0] = NULL;
+		TDS72_SMP_HEADER *p;
+
+		tds_extra_assert(tds->sid == 0);
+		tds_extra_assert(tds->conn->sessions[0] == tds);
+		tds_extra_assert(tds->send_packet != NULL);
+		tds_extra_assert(!tds->send_packet->next);
+
 		tds->conn->mars = 1;
-		tds->sid = -1;
+
+		/* start session with a SMP SYN */
+		if (TDS_FAILED(tds_append_syn(tds)))
+			return -TDSEMEM;
+
+		/* reallocate send_packet */
+		if (!tds_realloc_socket(tds, tds->out_buf_max))
+			return -TDSEMEM;
+
+		/* start SMP DATA header */
+		p = (TDS72_SMP_HEADER *) tds->send_packet->buf;
+		p->signature = TDS72_SMP;
+		p->type = TDS_SMP_DATA;
+
 		tds_init_write_buf(tds);
 	}
 #endif
@@ -642,10 +666,6 @@ tds_put_login_string(TDSSOCKET * tds, const char *buf, int n)
 static TDSRET
 tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 {
-#ifdef WORDS_BIGENDIAN
-	static const unsigned char be1[] = { 0x02, 0x00, 0x06, 0x04, 0x08, 0x01 };
-	static const unsigned char be2[] = { 0x00, 12, 16 };
-#endif
 	static const unsigned char le1[] = { 0x03, 0x01, 0x06, 0x0a, 0x09, 0x01 };
 	static const unsigned char le2[] = { 0x00, 13, 17 };
 
@@ -666,6 +686,8 @@ tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 
 	unsigned char protocol_version[4];
 	unsigned char program_version[4];
+	unsigned char sec_flags = 0;
+	bool use_kerberos = false;
 
 	int len;
 	char blockstr[16];
@@ -680,13 +702,27 @@ tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 		return TDS_FAIL;
 	}
 	if (tds_dstr_isempty(&login->user_name)) {
-		tdsdump_log(TDS_DBG_ERROR, "Kerberos login not supported using TDS 4.x or 5.0\n");
+		if (!IS_TDS50(tds->conn)) {
+			tdsdump_log(TDS_DBG_ERROR, "Kerberos login not supported using TDS 4.x\n");
+			return TDS_FAIL;
+		}
+
+#ifdef ENABLE_KRB5
+		/* try kerberos */
+		sec_flags = TDS5_SEC_LOG_SECSESS;
+		use_kerberos = true;
+		tds->conn->authentication = tds_gss_get_auth(tds);
+		if (!tds->conn->authentication)
+			return TDS_FAIL;
+#else
+		tdsdump_log(TDS_DBG_ERROR, "requested GSS authentication but not compiled in\n");
 		return TDS_FAIL;
+#endif
 	}
 	if (encryption_level == TDS_ENCRYPTION_DEFAULT)
 		encryption_level = TDS_ENCRYPTION_OFF;
-	if (encryption_level != TDS_ENCRYPTION_OFF) {
-		if (IS_TDS42(tds->conn)) {
+	if (!use_kerberos && encryption_level != TDS_ENCRYPTION_OFF) {
+		if (!IS_TDS50(tds->conn)) {
 			tdsdump_log(TDS_DBG_ERROR, "Encryption not supported using TDS 4.x\n");
 			return TDS_FAIL;
 		}
@@ -724,15 +760,7 @@ tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 	}
 	sprintf(blockstr, "%d", (int) getpid());
 	tds_put_login_string(tds, blockstr, TDS_MAXNAME);	/* host process */
-#ifdef WORDS_BIGENDIAN
-	if (tds->conn->emul_little_endian) {
-		tds_put_n(tds, le1, 6);
-	} else {
-		tds_put_n(tds, be1, 6);
-	}
-#else
 	tds_put_n(tds, le1, 6);
-#endif
 	tds_put_byte(tds, !login->bulk_copy);
 	tds_put_n(tds, NULL, 2);
 	if (IS_TDS42(tds->conn)) {
@@ -765,22 +793,16 @@ tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 	} else {
 		tds_put_n(tds, program_version, 4);	/* program version ? */
 	}
-#ifdef WORDS_BIGENDIAN
-	if (tds->conn->emul_little_endian) {
-		tds_put_n(tds, le2, 3);
-	} else {
-		tds_put_n(tds, be2, 3);
-	}
-#else
 	tds_put_n(tds, le2, 3);
-#endif
 	tds_put_login_string(tds, tds_dstr_cstr(&login->language), TDS_MAXNAME);	/* language */
 	tds_put_byte(tds, login->suppress_language);
 
 	/* oldsecure(2), should be zero, used by old software */
 	tds_put_n(tds, NULL, 2);
 	/* seclogin(1) bitmask */
-	tds_put_byte(tds, encryption_level != TDS_ENCRYPTION_OFF ? TDS5_SEC_LOG_ENCRYPT2|TDS5_SEC_LOG_NONCE : 0);
+	if (sec_flags == 0 && encryption_level != TDS_ENCRYPTION_OFF)
+		sec_flags = TDS5_SEC_LOG_ENCRYPT2|TDS5_SEC_LOG_ENCRYPT3;
+	tds_put_byte(tds, sec_flags);
 	/* secbulk(1)
 	 * halogin(1) type of ha login
 	 * hasessionid(6) id of session to reconnect
@@ -808,11 +830,18 @@ tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 	} else if (IS_TDS50(tds->conn)) {
 		/* just padding to 8 bytes */
 		tds_put_n(tds, NULL, 4);
+
+		/* send capabilities */
 		tds_put_byte(tds, TDS_CAPABILITY_TOKEN);
 		tds_put_smallint(tds, sizeof(tds->conn->capabilities));
 		tds_put_n(tds, &tds->conn->capabilities, sizeof(tds->conn->capabilities));
 	}
 
+#ifdef ENABLE_KRB5
+	if (use_kerberos)
+		tds5_gss_send(tds);
+#endif
+
 	return tds_flush_packet(tds);
 }
 
@@ -1042,10 +1071,8 @@ tds7_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 	/* host name */
 	PUT_STRING_FIELD_PTR(HOST_NAME);
 	if (tds->conn->authentication) {
-		tds_put_smallint(tds, 0);
-		tds_put_smallint(tds, 0);
-		tds_put_smallint(tds, 0);
-		tds_put_smallint(tds, 0);
+		tds_put_int(tds, 0);
+		tds_put_int(tds, 0);
 	} else {
 		/* username */
 		PUT_STRING_FIELD_PTR(USER_NAME);
@@ -1251,7 +1278,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 	tds->in_pos += pkt_len;
 	/* TODO some mssql version do not set last packet, update tds according */
 
-	tdsdump_log(TDS_DBG_INFO1, "detected flag %d\n", crypt_flag);
+	tdsdump_log(TDS_DBG_INFO1, "detected crypt flag %d\n", crypt_flag);
 
 	/* if server do not has certificate do normal login */
 	if (crypt_flag == TDS7_ENCRYPT_NOT_SUP) {
@@ -1263,7 +1290,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 	}
 
 	/*
-	 * if server has a certificate it require at least a crypted login
+	 * if server has a certificate it requires at least a crypted login
 	 * (even if data is not encrypted)
 	 */
 
@@ -1279,7 +1306,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 
 	ret = tds7_send_login(tds, login);
 
-	/* if flag is 0 it means that after login server continue not encrypted */
+	/* if flag is TDS7_ENCRYPT_OFF(0) it means that after login server continue not encrypted */
 	if (crypt_flag == TDS7_ENCRYPT_OFF || TDS_FAILED(ret))
 		tds_ssl_deinit(tds->conn);
 
