@@ -45,6 +45,7 @@
 #include <freetds/checks.h>
 #include <freetds/bytes.h>
 #include <freetds/alloca.h>
+#include <freetds/encodings.h>
 #include "replacements.h"
 
 /** \cond HIDDEN_SYMBOLS */
@@ -252,7 +253,7 @@ tds_process_default_tokens(TDSSOCKET * tds, int marker)
 		tok_size = tds_get_byte(tds);
 		if (tok_size >= 3) {
 			tds_get_byte(tds);
-			tds5_negotiate_set_msg_type(tds, tds->conn->authentication, tds_get_usmallint(tds));
+			tds5_negotiate_set_msg_type(tds->conn->authentication, tds_get_usmallint(tds));
 			tok_size -= 3;
 		}
 		tds_get_n(tds, NULL, tok_size);
@@ -386,7 +387,7 @@ tds_process_loginack(TDSSOCKET *tds, TDSRET *login_succeeded)
 	 * TDS 4.2 reports 1 on success and is not
 	 * present on failure
 	 */
-	if (ack == 5 || ack == 1) {
+	if (ack == 5 || ack == 1 || (IS_TDS50(tds->conn) && ack == 0x85)) {
 		*login_succeeded = TDS_SUCCESS;
 		/* authentication is now useless */
 		if (tds->conn->authentication) {
@@ -1519,6 +1520,7 @@ tds7_get_data_info(TDSSOCKET * tds, TDSCOLUMN * curcol)
 	curcol->column_nullable = curcol->column_flags & 0x01;
 	curcol->column_writeable = (curcol->column_flags & 0x08) > 0;
 	curcol->column_identity = (curcol->column_flags & 0x10) > 0;
+	curcol->column_computed = (curcol->column_flags & 0x20) > 0;
 
 	TDS_GET_COLUMN_TYPE(curcol);	/* sets "cardinal" type */
 
@@ -2194,7 +2196,6 @@ tds_process_env_chg(TDSSOCKET * tds)
 	char *newval = NULL;
 	char **dest;
 	int new_block_size;
-	int lcid;
 	int memrc = 0;
 
 	CHECK_TDS_EXTRA(tds);
@@ -2230,8 +2231,7 @@ tds_process_env_chg(TDSSOCKET * tds)
 		} else {
 			tds_get_n(tds, tds->conn->collation, 5);
 			tds_get_n(tds, NULL, size - 5);
-			lcid = TDS_GET_UA4LE(tds->conn->collation) & 0xffffflu;
-			tds7_srv_charset_changed(tds->conn, tds->conn->collation[4], lcid);
+			tds7_srv_charset_changed(tds->conn, tds->conn->collation);
 		}
 		tdsdump_dump_buf(TDS_DBG_NETWORK, "tds->conn->collation now", tds->conn->collation, 5);
 		/* discard old one */
@@ -3231,13 +3231,14 @@ adjust_character_column_size(TDSSOCKET * tds, TDSCOLUMN * curcol)
 	if (curcol->on_server.column_type == SYBLONGBINARY && (
 		curcol->column_usertype == USER_UNICHAR_TYPE ||
 		curcol->column_usertype == USER_UNIVARCHAR_TYPE)) {
+		const int canonic_client = tds->conn->char_convs[client2ucs2]->from.charset.canonic;
 #ifdef WORDS_BIGENDIAN
-		static const char sybase_utf[] = "UTF-16BE";
+		const int sybase_utf = TDS_CHARSET_UTF_16BE;
 #else
-		static const char sybase_utf[] = "UTF-16LE";
+		const int sybase_utf = TDS_CHARSET_UTF_16LE;
 #endif
 
-		curcol->char_conv = tds_iconv_get(tds->conn, tds->conn->char_convs[client2ucs2]->from.charset.name, sybase_utf);
+		curcol->char_conv = tds_iconv_get_info(tds->conn, canonic_client, sybase_utf);
 
 		/* fallback to UCS-2LE */
 		/* FIXME should be useless. Does not works always */
@@ -3245,8 +3246,7 @@ adjust_character_column_size(TDSSOCKET * tds, TDSCOLUMN * curcol)
 			curcol->char_conv = tds->conn->char_convs[client2ucs2];
 	}
 
-	/* FIXME: and sybase ?? */
-	if (!curcol->char_conv && IS_TDS7_PLUS(tds->conn) && is_ascii_type(curcol->on_server.column_type))
+	if (!curcol->char_conv && is_ascii_type(curcol->on_server.column_type))
 		curcol->char_conv = tds->conn->char_convs[client2server_chardata];
 
 	if (!USE_ICONV || !curcol->char_conv)
@@ -3282,6 +3282,11 @@ determine_adjusted_size(const TDSICONV * char_conv, int size)
 	if (!char_conv)
 		return size;
 
+	/* same charset */
+	if ((char_conv->flags & TDS_ENCODING_MEMCPY) != 0
+	    || char_conv->to.charset.canonic == char_conv->from.charset.canonic)
+		return size;
+
 	/* avoid possible overflow */
 	if (size >= 0x10000000)
 		return 0x7fffffff;
